package com.ahazou.photo_editor_sdk

import android.app.Activity
import android.app.AlertDialog
import android.app.ProgressDialog
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.net.Uri
import android.util.Log
import androidx.annotation.NonNull
import com.ahazou.photo_editor_sdk.service.ApiClient
import com.ahazou.photo_editor_sdk.service.ApiService
import com.ahazou.photo_editor_sdk.service.LogoToResponse
import com.bumptech.glide.Glide
import com.bumptech.glide.load.DataSource
import com.bumptech.glide.load.engine.GlideException
import com.bumptech.glide.request.RequestListener
import com.bumptech.glide.request.target.Target
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import ly.img.android.pesdk.PhotoEditorSettingsList
import ly.img.android.pesdk.assets.font.basic.FontPackBasic
import ly.img.android.pesdk.backend.decoder.ImageSource
import ly.img.android.pesdk.backend.layer.StickerGlLayer
import ly.img.android.pesdk.backend.layer.TextGlLayer
import ly.img.android.pesdk.backend.model.config.ImageStickerAsset
import ly.img.android.pesdk.backend.model.constant.Directory.DCIM
import ly.img.android.pesdk.backend.model.state.AssetConfig
import ly.img.android.pesdk.backend.model.state.LoadSettings
import ly.img.android.pesdk.backend.model.state.SaveSettings
import ly.img.android.pesdk.backend.model.state.manager.SettingsList
import ly.img.android.pesdk.ui.activity.ImgLyIntent
import ly.img.android.pesdk.ui.activity.PhotoEditorBuilder
import ly.img.android.pesdk.ui.model.state.UiConfigMainMenu
import ly.img.android.pesdk.ui.model.state.UiConfigSticker
import ly.img.android.pesdk.ui.model.state.UiConfigText
import ly.img.android.pesdk.ui.model.state.UiConfigTheme
import ly.img.android.pesdk.ui.panels.item.*
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.io.File
import java.io.FileOutputStream
import java.lang.ref.WeakReference
import java.util.*
import android.graphics.Bitmap















/** PhotoEditorSdkPlugin */
class PhotoEditorSdkPlugin:
        ActivityAware,
        FlutterPlugin,
        MethodCallHandler,
        PluginRegistry.ActivityResultListener {

  companion object {
    lateinit var activity: WeakReference<Activity>
    var arrLogos: ArrayList<CustomImageStickerAsset> = arrayListOf()
    var arrStickers: ArrayList<CustomImageStickerAsset> = arrayListOf()
    var userId = ""
    lateinit var resultClass: Result
    lateinit var progressDialog: ProgressDialog
    val SHARED_PREF = "sharedPreferences"
    lateinit var sharedPref: SharedPreferences
  }

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    val channel = MethodChannel(flutterPluginBinding.binaryMessenger, "photo_editor_sdk")
    channel.setMethodCallHandler(PhotoEditorSdkPlugin())
  }

  override fun onDetachedFromActivity() {}

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {}

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = WeakReference(binding.activity)
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {}

  override fun onDetachedFromActivityForConfigChanges() {}

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (activity.get() != null ) {
      val shared = activity.get()!!.getSharedPreferences(SHARED_PREF, Context.MODE_PRIVATE)
      sharedPref = shared
    }
    resultClass = result
    when ( call.method ) {
      "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
      "addAllContents" -> {
        val args = call.arguments as Map<*, *>
        val logos: ArrayList<*> = args["logos"] as ArrayList<*>
        val stickers: ArrayList<*> = args["stickers"] as ArrayList<*>
        arrLogos = retrieveAssets(logos)
        arrStickers = retrieveAssets(stickers)
      }
      "editImageByUrl" -> {
        val args = call.arguments as? String
        val uri = Uri.parse(args)
        openEditor(uri)
      }
      "editImageByFile" -> {
        val args = call.arguments as? String
        val file = File(args)
        val uri = Uri.fromFile(file)
        openEditor(uri)
      }
      "setUserId" -> {
        val id = call.arguments as String
        userId = id
      }
      "hasAddLogo" -> {
        val args = call.arguments as Map<*, *>
        val hasAddLogo = args["hasAddLogo"] as Boolean
        val urlAddLogo = args["urlAddLogo"] as String

        val editorShared = sharedPref.edit()
        editorShared.putBoolean("hasAddLogo", hasAddLogo)
        editorShared.putString("urlAddLogo", urlAddLogo)
        editorShared.apply()
      }
      else -> result.notImplemented()
    }
  }

  private fun retrieveAssets(list: ArrayList<*>): ArrayList<CustomImageStickerAsset> {
    val result = ArrayList<CustomImageStickerAsset>()
    list.forEach {
      Glide
        .with(activity.get()!!)
        .asFile()
        .load(it)
        .listener(object : RequestListener<File> {
          override fun onLoadFailed(e: GlideException?, model: Any?,
                                    target: Target<File>?, isFirstResource: Boolean):
                  Boolean {
            return false
          }

          override fun onResourceReady(resource: File?, model: Any?, target: Target<File>?,
                                       dataSource: DataSource?, isFirstResource: Boolean):
                  Boolean {

            val uri = Uri.fromFile(resource)

            val imageStickerConfig = ImageStickerAsset.createTemporaryStickerAsset(uri)
            result.add(CustomImageStickerAsset(imageStickerConfig, uri))
            return false
          }
        })
        .submit()
    }
    return result
  }

  private fun createPesdkSettingsList(inputImage: Uri?) =
      PhotoEditorSettingsList()
          .configure<UiConfigText> {
            it.setFontList(FontPackBasic.getFontPack())
          }
          .configure<SaveSettings> {
            it.setExportDir(DCIM, "AhazouPro")
            it.setExportPrefix("result_")
            it.savePolicy = SaveSettings.SavePolicy.RETURN_ALWAYS_ONLY_OUTPUT
          }
          .configure<LoadSettings> {
            it.source = inputImage
            it.isDeleteProtectedSource = true
          }
          .configure<UiConfigTheme> {
            it.theme =  R.style.Imgly_Theme_TopActionBar_NoFullscreen
          }
          .configure<UiConfigMainMenu> {
            it.setToolList(
                    ToolItem("imgly_tool_sticker_selection",
                            R.string.pesdk_sticker_title_name,
                            ImageSource.create(R.drawable.imgly_icon_tool_sticker)),
                    ToolItem("imgly_tool_text",
                            R.string.pesdk_text_title_name,
                            ImageSource.create(R.drawable.imgly_icon_tool_text))
            )
          }
          .configure<UiConfigSticker> {

            StickerGlLayer.SORTED_ROTATION_SNAP_POINTS = FloatArray(6)
            StickerGlLayer.SNAP_PADDING_LEFT = 0.0f
            StickerGlLayer.SNAP_PADDING_TOP = 0.0f
            StickerGlLayer.SNAP_PADDING_RIGHT = 0.0f
            StickerGlLayer.SNAP_PADDING_BOTTOM = 0.0f

            TextGlLayer.SORTED_ROTATION_SNAP_POINTS = FloatArray(6)
            TextGlLayer.SNAP_PADDING_LEFT = 0.0f
            TextGlLayer.SNAP_PADDING_TOP = 0.0f
            TextGlLayer.SNAP_PADDING_RIGHT = 0.0f
            TextGlLayer.SNAP_PADDING_BOTTOM = 0.0f

            val arrLogoSticker = ArrayList<AbstractIdItem>()

            // Add-logo button
            if (sharedPref.getBoolean("hasAddLogo", false)) {
              arrLogoSticker.add(PersonalStickerAddItem("imgly_add_personal_sticker",
                      R.string.pesdk_sticker_button_add,
                      ImageSource.create(R.drawable.ic_add_logo)))
            }

            // Stickers & logos
            if( arrStickers.isNotEmpty() ) {
              arrLogoSticker.add(StickerCategoryItem(
                      "stickers_id",
                      "stickers",
                      ImageSource.create(R.mipmap.ic_sticker),
                      arrStickers.map { s -> ImageStickerItem.createFromAsset(s.image) }
              ))
              arrLogoSticker.add(StickerCategoryItem(
                      "imgly_add_personal_sticker",
                      "logos",
                      ImageSource.create(R.mipmap.ic_logo),
                      arrLogos.map { s -> ImageStickerItem.createFromAsset(s.image) }
              ))
            }
            it.setStickerLists(arrLogoSticker)
          }
          .configure<AssetConfig> {
            arrLogos.map { logo -> it.addAsset(logo.image) }
            arrStickers.map { sticker -> it.addAsset(sticker.image) }
          }

  val PESDK_RESULT = 1000

  private fun openEditor(inputImage: Uri?) {
    val settingsList = createPesdkSettingsList(inputImage)

    Thread {
      PhotoEditorBuilder(activity.get())
              .setSettingsList(settingsList)
              .startActivityForResult(activity.get(), PESDK_RESULT)
    }.run()
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent): Boolean {
      if (resultCode == Activity.RESULT_OK && requestCode == PESDK_RESULT) {
        val settings = data.getParcelableExtra<SettingsList?>(ImgLyIntent.SETTINGS_LIST)
        val model = settings?.getSettingsModel(AssetConfig::class.java)
        val mapAsset = model!!.getAssetMap(ImageStickerAsset::class.java)
        val arrLogosSticker = ArrayList<ImageStickerAsset>()
        arrLogos.map { logos -> arrLogosSticker.add(logos.image) }
        arrStickers.map { logos -> arrLogosSticker.add(logos.image) }

        val dynamicStickers = ArrayList<ImageStickerAsset>()
        mapAsset.map { map -> dynamicStickers.add(map) }

        if(dynamicStickers.size > arrLogosSticker.size) {
          if (activity.get() != null) {
            if (!activity.get()!!.isFinishing) {
              progressDialog = ProgressDialog(activity.get()!!)
              progressDialog.setMessage("Enviando dados...")
              progressDialog.setCancelable(false)
              progressDialog.show()
            }
          }

          val newLogo = ArrayList<ImageStickerAsset>()

          for (new in dynamicStickers) {
            var found = false
            for (logo in arrLogosSticker) {
              if (new.id == logo.id) {
                found = true
                break
              }
            }
            if (!found) newLogo.add(new)
          }

          newLogo.forEachIndexed { i, logo -> Thread {
            if (logo.stickerSource.bitmap != null) {
              val resized = resize(logo.stickerSource.bitmap!!, 300, 300)
              uploadLogo(persistLogo(resized), i == newLogo.size - 1, data)
            }
          }.start()}
      } else {
        savePost(data)
      }
      return true
    }
    return false
  }

  private fun resize(image: Bitmap, maxWidth: Int, maxHeight: Int): Bitmap {
    val image: Bitmap = image
    if (maxHeight > 0 && maxWidth > 0) {
      val width: Int = image.getWidth()
      val height: Int = image.getHeight()
      val ratioBitmap = width.toFloat() / height.toFloat()
      val ratioMax = maxWidth.toFloat() / maxHeight.toFloat()
      var finalWidth = maxWidth
      var finalHeight = maxHeight
      if (ratioMax > ratioBitmap) {
        finalWidth = (maxHeight.toFloat() * ratioBitmap).toInt()
      } else {
        finalHeight = (maxWidth.toFloat() / ratioBitmap).toInt()
      }
      return Bitmap.createScaledBitmap(image, finalWidth, finalHeight, true)
    } else {
      return image
    }
  }

  private fun savePost(data: Intent) {
    data.getParcelableExtra<Uri?>(ImgLyIntent.RESULT_IMAGE_URI)?.also {
      activity.get()!!.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE).setData(it))
      val filePath = it.path
      val file = File(filePath)
      val b = file.readBytes()
      resultClass.success(b)
    }
  }

  private fun persistLogo(bitmap: Bitmap, name: String = "temporary_logo"): File {
    val filesDir = activity.get()!!.applicationContext.filesDir
    val imageFile = File(filesDir, "$name.png")

    try {
      val os = FileOutputStream(imageFile)
      bitmap.compress(Bitmap.CompressFormat.PNG, 100, os)
      os.flush()
      os.close()
    } catch (e: Exception) {
      Log.e(this.javaClass.simpleName, "Error writing bitmap", e)
    }
    return imageFile
  }

  private fun uploadLogo(file: File, lastOne: Boolean, data: Intent) {
    val requestBody = RequestBody.create(MediaType.parse("*/*"), file)
    val fileToUpload = MultipartBody.Part.createFormData("logo", file.name, requestBody)
    var multipartUserId = MultipartBody.Part.createFormData("userId", userId)

    ApiClient.BASE_URL = sharedPref.getString("urlAddLogo", "")
    val getResponse = ApiClient.getClient().create(ApiService::class.java)
    val call = getResponse.addLogo(multipartUserId, fileToUpload)

    call.enqueue(object : Callback<LogoToResponse> {
      override fun onFailure(call: Call<LogoToResponse>, t: Throwable) {
        progressDialog.dismiss()
        if (activity.get() != null ) {
          val builder = AlertDialog.Builder(activity.get()!!, AlertDialog.THEME_DEVICE_DEFAULT_LIGHT)
          builder.setTitle("ops!")
          builder.setMessage("não conseguimos salvar sua logo. quer tentar novamente?")
          builder.setPositiveButton("ok") { dialog, which -> }
          val dialog: AlertDialog = builder.create()

          dialog.show()
        }
      }

      override fun onResponse(call: Call<LogoToResponse>, response: Response<LogoToResponse>) {
        progressDialog.dismiss()

        if( lastOne ) savePost(data)
      }
    })
  }

  data class CustomImageStickerAsset(val image: ImageStickerAsset, val uri: Uri)
}
