//
//  UploadImageManager.swift
//  flutter_plugin_android_lifecycle
//
//  Created by Danilo Pena on 29/05/20.
//

import UIKit

class UploadImageManager: NSObject {
    let formFields = ["name": "ahazou", "family_name": "pro"]
    var imageData: Data? = nil

    func convertFormField(named name: String, value: String, using boundary: String) -> String {
      var fieldString = "--\(boundary)\r\n"
      fieldString += "Content-Disposition: form-data; name=\"\(name)\"\r\n"
      fieldString += "\r\n"
      fieldString += "\(value)\r\n"

      return fieldString
    }

    func convertFileData(fieldName: String, fileName: String, mimeType: String, fileData: Data, using boundary: String) -> Data {
      let data = NSMutableData()

      data.appendString("--\(boundary)\r\n")
      data.appendString("Content-Disposition: form-data; name=\"\(fieldName)\"; filename=\"\(fileName)\"\r\n")
      data.appendString("Content-Type: \(mimeType)\r\n\r\n")
      data.append(fileData)
      data.appendString("\r\n")

      return data as Data
    }
    
    func makeRequest() -> URLRequest? {
        guard let urlStr = UserDefaults.standard.string(forKey: "urlAddLogo"),
              let url = URL(string: urlStr) else {
                return nil
        }
        let boundary = "Boundary-\(UUID().uuidString)"
        
        var request = URLRequest(url: url)
        request.httpMethod = "POST"
        request.setValue("multipart/form-data; boundary=\(boundary)", forHTTPHeaderField: "Content-Type")

        let httpBody = NSMutableData()
        for (key, value) in formFields {
          httpBody.appendString(convertFormField(named: key, value: value, using: boundary))
        }
        httpBody.appendString("--\(boundary)\r\n")
        httpBody.appendString("Content-Disposition:form-data; name=userId")
        httpBody.appendString("\r\n\r\n\(UserDefaults.standard.string(forKey: "userId")!)\r\n")

        httpBody.append(convertFileData(fieldName: "logo",
                                        fileName: "image.png",
                                        mimeType: "file",
                                        fileData: imageData ?? Data(),
                                        using: boundary))

        httpBody.appendString("--\(boundary)--")

        request.httpBody = httpBody as Data
        
        return request
    }
}

extension NSMutableData {
  func appendString(_ string: String) {
    if let data = string.data(using: .utf8) {
      self.append(data)
    }
  }
}
